__title__ = 'pypelined'
__summary__ = 'stream and pipeline processing service'
__url__ = 'https://github.com/maxfischer2781/pypelined'

__version__ = '0.1.3'
__author__ = 'Max Fischer'
__email__ = 'maxfischer2781@gmail.com'
__copyright__ = '2017 %s' % __author__
